<?php
/*******************************************************************************
*    myMVC - Model-View-Controller for PHP                                     *
*    Copyright (C) 2012 Luca Bertoncello                                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU Lesser General Public License as published  *
*    by the Free Software Foundation; version 2.1 of the License.              *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU Lesser General Public License for more details.                       *
*                                                                              *
*    You should have received a copy of the GNU Lesser General Public License  *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

/**
 * Filter class to implement a login system.
 * It checks if a user is authenticated and, if not, redirect
 * to a login action (derived from myMVC_BaseLoginAction)
 */
class myMVC_LoginFilter extends myMVC_BaseFilter
{
  /**
   * Constructor.
   * 
   * @param myMVC_HttpRequest $request      The Request
   * @param myMVC_HttpSession $session      The session
   * @param class $calledClass              The called class (subclass of myMVC_BaseAction or myMVC_BaseView)
   * @param string $filterType              Filter type (PREACTION or PREVIEW)
   * @param array $param                    The optional parameters of the filter
   */
  public function __construct($request, $session, $calledClass, $filterType, $param = array())
  {
    if($param['loginAction'] === NULL)
    {
      $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, "No login action class given (parameter 'loginAction'). Aborting");
      throw new myMVC_MVCException($msg);
    }
    $loginAction = $param['loginAction'];
    if(!is_subclass_of($loginAction, 'myMVC_BaseLoginAction'))
    {
      $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Action class %s is not a subclass of myMVC_BaseLoginAction", $loginAction));
      throw new myMVC_MVCException($msg);
    }
    parent::__construct($request, $session, $calledClass, $filterType, $param);
  }

  /**
   * Check if the user already logged in.
   * If not, redirect to the login action (given as parameter)
   * 
   * @return myMVC_HttpResponse             Response code for an Action or NULL, if the program should continue
   */
  public function filter()
  {
    $user = $this->session->getFromNamespace('LOGIN', 'loggedUser');
    if($user !== NULL && is_subclass_of($user, 'myMVC_BaseLogin'))
      return NULL;

    if(is_subclass_of($this->calledClass, 'myMVC_BaseLoginAction') ||
       is_subclass_of($this->calledClass, 'myMVC_BaseLoginView'))
      return NULL;

    $loginAction = $this->param['loginAction'];
    myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("User not logged in. Redirecting to the login action %s", $loginAction), ALERT_DEBUG);
    $response = new myMVC_HttpResponse();
    $response->setNext($loginAction);
    if(($this->request->getQueryType() == myMVC_HttpRequest::$TYPE_ACTION ||
        $this->request->getQueryType() == myMVC_HttpRequest::$TYPE_MENU) &&
       (!$this->session->existsInNamespace('LOGIN', 'nextAction')))
    {
      $action = str_replace(array_merge((array)('&'.$this->session->getSessionId()),
                                        myMVC_HttpRequest::getQueryTypes('=')),
                            array_fill(0, count(myMVC_HttpRequest::getQueryTypes()) + 2, ''),
                            $this->request->getQueryString());
      $this->session->setInNamespace('LOGIN', 'nextAction', $action);
      $this->session->setInNamespace('LOGIN', 'nextType', $this->request->getQueryType());
    }
    return $response;
  }
}
?>
