<?php
class MainAction extends GUIAction
{
  public function __construct($request, $session)
  {
    parent::__construct($request, $session);
    $this->setCheckFunction('GET', 'validateGet');
    $this->setCheckFunction('POST', 'validatePost');
  }

  /**
   * Returns the CSS class of this action
   * 
   * @return string                     The CSS class
   */
  public static function getCSSClass()
  {
    return 'general';
  }

  /**
   * Returns the token used to identify the current Action
   * 
   * @return string                 The token
   */
  public static function getActionToken()
  {
    return 'main';
  }

  /**
   * Returns whether this action should appear in the menu (created by controller)
   * 
   * @return boolean                true if the action should appear in menu
   */
  public static function isMenuAction()
  {
    return true;
  }

  /**
   * Returns the position of this action in the menu (created by controller)
   * Only necessary if isMenuAction() returns true
   * 
   * @return int                        The position in the menu (lower is "upper")
   */
  public static function getActionMenuPosition()
  {
    return 10;
  }

  /**
   * Returns the string to be displayed in the menu to identify the current action.
   * Only necessary if isMenuAction() returns true
   * 
   * @return string                 The menu item
   */
  public static function getActionMenuName()
  {
    return _("Hauptseite");
  }

  /**
   * Gibt den Name des Bildes für dieses Menüpunkt zurücl
   * 
   * @return string                 The menu item
   */
  public static function getActionMenuImage()
  {
    return '';
  }

  /**
   * Returns whether the current user is allowed to call the current Action
   * 
   * @return boolean                true if the user may call this Action, false otherwise
   */
  public static function hasUserRightForAction()
  {
    return true;
  }
  
  /**
   * Validate the data from user, coming as GET
   *
   * @return boolean                true if data are valid, false otherwise
   */
  public function validateGet()
  {
    $user = self::getLoggedUser();
    $this->session->useNamespace('main');
    if($user->getIsAdmin() != 'f')
      $acc = $this->session->get('account');
    else
      $acc = Account::findByPK(array('accountname' => $user->getAccountname()));

    if($this->request->getSubmitParameter('deletefwd') != '')
    {
      $fwds = array();
      foreach(explode("\n", str_replace('deliver ', '', $acc->getDataHam())) as $f)
        if($f != '')
        array_push($fwds, $f);
      $definitions = array(
          'deletefwd'           => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => $fwds),
      );
    }
    else
      return true;

    $names = array(
        'deletefwd'             => _('Weiterleitungsziel'),
    );
    $errorMsgs = array(
        'deletefwd'             => array(myMVC_BaseAction::$ERROR_EMPTY         => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                         myMVC_BaseAction::$ERROR_RANGE         => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                         myMVC_BaseAction::$ERROR_NOTSTRING     => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
    );
    return parent::validateInput($definitions, $names, $errorMsgs);
  }

  /**
   * Validate the data from user, coming as POST
   *
   * @return boolean                true if data are valid, false otherwise
   */
  public function validatePost()
  {
    global $config;

    $user = self::getLoggedUser();
    if($this->request->getSubmitParameter('action') == 'chgAdminUser')
    {
      $users = array('__SELF__');
      foreach($user->getUsersForAdmin() as $u)
        array_push($users, $u->getAccountname());
      $definitions = array(
          'adminUser'           => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => $users),
      );
    }
    elseif($this->request->getSubmitParameter('action') == 'changePipe')
    {
      $pipes = array();
      foreach($config['MBOXCONFIG']['PIPES'] as $pipe)
        array_push($pipes, $pipe['name']);
      $definitions = array(
          'pipeName'            => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => $pipes),
      );
    }
    elseif($this->request->getSubmitParameter('chgPasswd') != '')
    {
      $definitions = array(
          'passwd'              => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'maxlen' => 32, 'minlen' => 5, 'empty' => false),
          'passwd2'             => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'maxlen' => 32, 'minlen' => 5, 'empty' => false,
                                                'comparefield' => array('field' => 'passwd', 'compareType' => '==')),
      );
    }
    elseif($this->request->getSubmitParameter('chgPipe') != '')
    {
      $pipes = array();
      foreach($config['MBOXCONFIG']['PIPES'] as $pipe)
        array_push($pipes, $pipe['name']);
      $definitions = array(
          'pipeName'            => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => $pipes),
          'pipeParameter'       => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false,
                  'range' => Account::getParametersForPipe($this->request->getSubmitParameter('pipeName'))),
      );
    }
    elseif($this->request->getSubmitParameter('newForward') != '')
    {
      $definitions = array(
          'newFwd'              => array('type' => myMVC_BaseAction::$VALIDATE_EMAIL, 'empty' => false),
      );
    }
    elseif($this->request->getSubmitParameter('save') != '')
    {
      $definitions = array(
        'spamScanType'          => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => array_keys(Account::getSpamScanTypes())),
        'rejectedSpamScore'     => array('type' => myMVC_BaseAction::$VALIDATE_NUMBER, 'empty' => true, 'min' => 1),
        'virusScanType'         => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => array_keys(Account::getVirusScanTypes())),
        'rejectAfterSize'       => array('type' => myMVC_BaseAction::$VALIDATE_NUMBER, 'empty' => true, 'min' => 1),
        'useSPF'                => array('type' => myMVC_BaseAction::$VALIDATE_BOOLEAN, 'empty' => false),
        'useDomainKey'          => array('type' => myMVC_BaseAction::$VALIDATE_BOOLEAN, 'empty' => false),
        'useGreylist'           => array('type' => myMVC_BaseAction::$VALIDATE_BOOLEAN, 'empty' => false),
      );
    }

    $names = array(
        'passwd'                => _('Passwort'),
        'passwd2'               => _('Passwortbest&auml;tigung'),
        'spamScanType'          => _('Spamschutz'),
        'rejectedSpamScore'     => _('Erweiterter Spamschutz'),
        'virusScanType'         => _('Schutz vor Schadsoftware'),
        'rejectAfterSize'       => _('Filter nach Gr&ouml;&szlig;e'),
        'useSPF'                => _('SPF-Pr&uuml;fung'),
        'useDomainKey'          => _('DomainKey-Pr&uuml;fung'),
        'useGreylist'           => _('Greylist'),
        'adminUser'             => _('Administrierter Nutzer'),
        'newFwd'                => _('Weiterleitungsziel'),
        'pipeName'              => _('Programm'),
        'pipeParameter'         => _('Parameter'),
    );
    $errorMsgs = array(
        'passwd'                => array(myMVC_BaseAction::$ERROR_MAXLEN        => sprintf(_("%s ist zu lang"), '{field}'),
                                         myMVC_BaseAction::$ERROR_MINLEN        => sprintf(_("%s ist zu kurz"), '{field}'),
                                         myMVC_BaseAction::$ERROR_EMPTY         => sprintf(_("%s kann nicht leer sein"), '{field}')),
        'passwd2'               => array(myMVC_BaseAction::$ERROR_MAXLEN        => sprintf(_("%s ist zu lang"), '{field}'),
                                         myMVC_BaseAction::$ERROR_MINLEN        => sprintf(_("%s ist zu kurz"), '{field}'),
                                         myMVC_BaseAction::$ERROR_EMPTY         => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                         myMVC_BaseAction::$ERROR_COMPAREFIELD  => _("Passwort und Passwortbest&auml;tigung sind nicht gleich")),
        'spamScanType'          => array(myMVC_BaseAction::$ERROR_EMPTY         => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                         myMVC_BaseAction::$ERROR_RANGE         => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                         myMVC_BaseAction::$ERROR_NOTSTRING     => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
        'rejectedSpamScore'     => array(myMVC_BaseAction::$ERROR_NOTNUMBER     => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                         myMVC_BaseAction::$ERROR_MIN           => sprintf(_("%s muss gr&ouml;&szlig;er als 1 sein"), '{field}')),
        'virusScanType'         => array(myMVC_BaseAction::$ERROR_EMPTY         => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                         myMVC_BaseAction::$ERROR_RANGE         => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                         myMVC_BaseAction::$ERROR_NOTSTRING     => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
        'rejectAfterSize'       => array(myMVC_BaseAction::$ERROR_NOTNUMBER     => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                         myMVC_BaseAction::$ERROR_MIN           => sprintf(_("%s muss gr&ouml;&szlig;er als 1 sein"), '{field}')),
        'useSPF'                => array(myMVC_BaseAction::$ERROR_EMPTY         => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                         myMVC_BaseAction::$ERROR_NOTBOOL       => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
        'useDomainKey'          => array(myMVC_BaseAction::$ERROR_EMPTY         => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                         myMVC_BaseAction::$ERROR_NOTBOOL       => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
        'useGreylist'           => array(myMVC_BaseAction::$ERROR_EMPTY         => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                         myMVC_BaseAction::$ERROR_NOTBOOL       => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
        'adminUser'             => array(myMVC_BaseAction::$ERROR_EMPTY         => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                         myMVC_BaseAction::$ERROR_RANGE         => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                         myMVC_BaseAction::$ERROR_NOTSTRING     => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
        'newFwd'                => array(myMVC_BaseAction::$ERROR_EMPTY         => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                         myMVC_BaseAction::$ERROR_NOTEMAIL      => sprintf(_("%s ist keine g&uuml;ltige E-Mail-Adresse"), '{field}')),
        'pipeName'              => array(myMVC_BaseAction::$ERROR_EMPTY         => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                         myMVC_BaseAction::$ERROR_RANGE         => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                         myMVC_BaseAction::$ERROR_NOTSTRING     => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
        'pipeParameter'         => array(myMVC_BaseAction::$ERROR_EMPTY         => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                         myMVC_BaseAction::$ERROR_RANGE         => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                         myMVC_BaseAction::$ERROR_NOTSTRING     => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
    );
    return parent::validateInput($definitions, $names, $errorMsgs);
  }

  /**
   * Perform the Action
   * 
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function perform()
  {
    global $config;

    $response = new myMVC_HttpResponse();

    $response->setNext('MainView');
    $this->session->useNamespace('main');
    $user = self::getLoggedUser();

    if($this->request->getRequestMethod() == 'GET')
    {
      if($this->request->getSubmitParameter('deletefwd') != '')
      {
        if($user->getIsAdmin() != 'f')
          $acc = $this->session->get('account');
        else
          $acc = Account::findByPK(array('accountname' => $user->getAccountname()));

        $fwds = array();
        $fwdstr = '';
        foreach(explode("\n", str_replace('deliver ', '', $acc->getDataHam())) as $f)
        {
          if($f != '')
          {
            if($f != $this->request->getParameter('deletefwd'))
            {
              array_push($fwds, $f);
              $fwdstr .= sprintf("deliver %s\n", $f);
            }
          }
        }
        $acc->setDataHam($fwdstr);
        $acc->save();
        $this->saveParameterForNextPage('forwards', $fwds);
      }
      else
      {
        $acc = Account::findByPK(array('accountname' => $user->getAccountname()));
        $this->session->set('account', $acc);
        $this->saveParameterForNextPage('spamScanType', $acc->getSpamScanType());
        $this->saveParameterForNextPage('virusScanType', $acc->getVirusScanType());
        $this->saveParameterForNextPage('useSPF', $acc->getUseSPF());
        $this->saveParameterForNextPage('useGreylist', $acc->getUseGreylist());
        $this->saveParameterForNextPage('useDomainKey', $acc->getUseDomainKey());
        $this->saveParameterForNextPage('rejectedSpamScore', $acc->getRejectedSpamScore() / 10);
        $this->saveParameterForNextPage('rejectAfterSize', $acc->getRejectAfterSize());
      }
    }
    else
    {
      if($this->request->getSubmitParameter('action') == 'chgAdminUser')
      {
        if($this->request->getParameter('adminUser') != '__SELF__')
          $acc = Account::findByPK(array('accountname' => $this->request->getParameter('adminUser')));
        else
          $acc = Account::findByPK(array('accountname' => $user->getAccountname()));
        $this->session->set('account', $acc);
        if(substr($acc->getDataHam(), 0, 8) == 'deliver ')
        {
          $fwds = array();
          foreach(explode("\n", str_replace('deliver ', '', $acc->getDataHam())) as $f)
            if($f != '')
              array_push($fwds, $f);
          $this->saveParameterForNextPage('forwards', $fwds);
        }
        elseif(substr($acc->getDataHam(), 0, 5) == 'pipe ')
        {
          $p = str_replace('pipe "', '', $acc->getDataHam());
          $p = str_replace('"', '', $p);
          list($pipeCmd, $pipeParameter) = explode(' ', $p);
          foreach($config['MBOXCONFIG']['PIPES'] as $pipe)
          {
            if($pipe['command'] == $pipeCmd)
            {
              $pipeName = $pipe['name'];
              list($type, $v) = explode(':', $pipe['values']);
              list($prefix, $other) = explode('|', $v);
              if($other == '' && $prefix != '')
              {
                $other = $prefix;
                $prefix = '';
              }
            }
          }
          $this->saveParameterForNextPage('pipeName', $pipeName);
          $this->saveParameterForNextPage('pipeParameter', str_replace($prefix, '', $pipeParameter));
        }
        $this->saveParameterForNextPage('spamScanType', $acc->getSpamScanType());
        $this->saveParameterForNextPage('virusScanType', $acc->getVirusScanType());
        $this->saveParameterForNextPage('useSPF', $acc->getUseSPF());
        $this->saveParameterForNextPage('useGreylist', $acc->getUseGreylist());
        $this->saveParameterForNextPage('useDomainKey', $acc->getUseDomainKey());
        $this->saveParameterForNextPage('rejectedSpamScore', $acc->getRejectedSpamScore() / 10);
        $this->saveParameterForNextPage('rejectAfterSize', $acc->getRejectAfterSize());
      }
      elseif($this->request->getSubmitParameter('action') == 'changePipe')
      {
        if($user->getIsAdmin() != 'f')
          $acc = $this->session->get('account');
        else
          $acc = Account::findByPK(array('accountname' => $user->getAccountname()));
        $this->saveParameterForNextPage('pipeName', $this->request->getParameter('pipeName'));
      }
      elseif($this->request->getSubmitParameter('chgPipe') != '')
      {
        if($user->getIsAdmin() != 'f')
          $acc = $this->session->get('account');
        else
          $acc = Account::findByPK(array('accountname' => $user->getAccountname()));
        foreach($config['MBOXCONFIG']['PIPES'] as $pipe)
        {
          if($pipe['name'] == $this->request->getParameter('pipeName'))
          {
            list($type, $v) = explode(':', $pipe['values']);
            list($prefix, $other) = explode('|', $v);
            if($other == '' && $prefix != '')
            {
              $other = $prefix;
              $prefix = '';
            }
            $acc->setDataHam(sprintf('pipe "%s %s%s"',
                                $pipe['command'], $prefix, $this->request->getParameter('pipeParameter')));
          }
        }
        $acc->save();
        $this->saveParameterForNextPage('pipeName', $this->request->getParameter('pipeName'));
        $this->saveParameterForNextPage('pipeParameter', $this->request->getParameter('pipeParameter'));
      }
      elseif($this->request->getSubmitParameter('newForward') != '')
      {
        if($user->getIsAdmin() != 'f')
          $acc = $this->session->get('account');
        else
          $acc = Account::findByPK(array('accountname' => $user->getAccountname()));
        
        $fwds = array();
        $fwdstr = '';
        foreach(explode("\n", str_replace('deliver ', '', $acc->getDataHam())) as $f)
        {
          if($f != '')
          {
            array_push($fwds, $f);
            $fwdstr .= sprintf("deliver %s\n", $f);
          }
        }
        array_push($fwds, $this->request->getParameter('newFwd'));
        $fwdstr .= sprintf("deliver %s\n", $this->request->getParameter('newFwd'));
        $acc->setDataHam($fwdstr);
        $acc->save();
        $this->saveParameterForNextPage('forwards', $fwds);
      }
      elseif($this->request->getSubmitParameter('chgPasswd') != '')
      {
        if($user->getIsAdmin() != 'f')
          $acc = $this->session->get('account');
        else
          $acc = Account::findByPK(array('accountname' => $user->getAccountname())); 
        $acc->setPasswd($this->request->getParameter('passwd'));
        $acc->save();

        $this->saveParameterForNextPage('spamScanType', $acc->getSpamScanType());
        $this->saveParameterForNextPage('virusScanType', $acc->getVirusScanType());
        $this->saveParameterForNextPage('useSPF', $acc->getUseSPF());
        $this->saveParameterForNextPage('useGreylist', $acc->getUseGreylist());
        $this->saveParameterForNextPage('useDomainKey', $acc->getUseDomainKey());
        $this->saveParameterForNextPage('rejectedSpamScore', $acc->getRejectedSpamScore() / 10);
        $this->saveParameterForNextPage('rejectAfterSize', $acc->getRejectAfterSize());
        $this->saveParameterForNextPage('popupMessage', _('Das Passwort wurde ge&auml;ndert'));
      }
      elseif($this->request->getSubmitParameter('save') != '')
      {
        if($user->getIsAdmin() != 'f')
          $acc = $this->session->get('account');
        else
          $acc = Account::findByPK(array('accountname' => $user->getAccountname()));
        $acc->setSpamScanType($this->request->getParameter('spamScanType'));
        $acc->setVirusScanType($this->request->getParameter('virusScanType'));
        $acc->setUseSPF($this->request->getParameter('useSPF'));
        $acc->setUseGreylist($this->request->getParameter('useGreylist'));
        $acc->setUseDomainKey($this->request->getParameter('useDomainKey'));
        if($this->request->getParameter('rejectedSpamScore') == '')
          $acc->setRejectedSpamScore(0);
        else
          $acc->setRejectedSpamScore($this->request->getParameter('rejectedSpamScore') * 10);
        if($this->request->getParameter('rejectAfterSize') == '')
          $acc->setRejectAfterSize(NULL);
        else
          $acc->setRejectAfterSize($this->request->getParameter('rejectAfterSize'));
        $acc->save();

        $this->saveParameterForNextPage('spamScanType', $acc->getSpamScanType());
        $this->saveParameterForNextPage('virusScanType', $acc->getVirusScanType());
        $this->saveParameterForNextPage('useSPF', $acc->getUseSPF());
        $this->saveParameterForNextPage('useGreylist', $acc->getUseGreylist());
        $this->saveParameterForNextPage('useDomainKey', $acc->getUseDomainKey());
        $this->saveParameterForNextPage('rejectedSpamScore', $acc->getRejectedSpamScore() / 10);
        $this->saveParameterForNextPage('rejectAfterSize', $acc->getRejectAfterSize());
        $this->saveParameterForNextPage('popupMessage', _('Die Einstellungen wurden erfolgreich gespeichert'));
      }
    }
    if($user->getIsAdmin() != 'f')
      $this->saveParameterForNextPage('adminUser', $acc->getAccountname());

    return $response;
  }

  /**
   * Called if user data are not valid (check with validate)
   * 
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function manageInvalidData()
  {
    $response = new myMVC_HttpResponse();
    $response->setNext('MainView');
    $this->saveParameterForNextPage('error', true);

    $user = self::getLoggedUser();
    $this->session->useNamespace('main');
    if($user->getIsAdmin() != 'f')
    {
      $acc = $this->session->get('account');
      $this->saveParameterForNextPage('adminUser', $acc->getAccountname());
    }
    else
      $acc = Account::findByPK(array('accountname' => $user->getAccountname()));

    if(substr($acc->getDataHam(), 0, 8) == 'deliver ')
    {
      $fwds = array();
      foreach(explode("\n", str_replace('deliver ', '', $acc->getDataHam())) as $f)
        if($f != '')
        array_push($fwds, $f);
      $this->saveParameterForNextPage('forwards', $fwds);
    }
    if($this->request->getSubmitParameter('chgPipe') != '')
    {
      $this->saveParameterForNextPage('pipeName', $this->request->getSubmitParameter('pipeName'));
      $this->saveParameterForNextPage('pipeParameter', $this->request->getSubmitParameter('pipeParameter'));
    }
    if($this->request->getSubmitParameter('chgPasswd') != '')
    {
      $this->saveParameterForNextPage('spamScanType', $acc->getSpamScanType());
      $this->saveParameterForNextPage('virusScanType', $acc->getVirusScanType());
      $this->saveParameterForNextPage('useSPF', $acc->getUseSPF());
      $this->saveParameterForNextPage('useGreylist', $acc->getUseGreylist());
      $this->saveParameterForNextPage('useDomainKey', $acc->getUseDomainKey());
      $this->saveParameterForNextPage('rejectedSpamScore', $acc->getRejectedSpamScore() / 10);
      $this->saveParameterForNextPage('rejectAfterSize', $acc->getRejectAfterSize());
    }

    return $response;
  }
}
?>
